@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,7 +25,7 @@
 
 #include "clientcommon.hpp"
 
-template <bool FORTRAN, typename T, typename U>
+template <testAPI_t API, typename T, typename U>
 void orgqr_ungqr_checkBadArgs(const hipsolverHandle_t handle,
                               const int               m,
                               const int               n,
@@ -39,7 +39,7 @@ void orgqr_ungqr_checkBadArgs(const hipsolverHandle_t handle,
 {
     // handle
     EXPECT_ROCBLAS_STATUS(
-        hipsolver_orgqr_ungqr(FORTRAN, nullptr, m, n, k, dA, lda, dIpiv, dWork, lwork, dInfo),
+        hipsolver_orgqr_ungqr(API, nullptr, m, n, k, dA, lda, dIpiv, dWork, lwork, dInfo),
         HIPSOLVER_STATUS_NOT_INITIALIZED);
 
     // values
@@ -48,19 +48,18 @@ void orgqr_ungqr_checkBadArgs(const hipsolverHandle_t handle,
 #if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // pointers
     EXPECT_ROCBLAS_STATUS(
-        hipsolver_orgqr_ungqr(
-            FORTRAN, handle, m, n, k, (T) nullptr, lda, dIpiv, dWork, lwork, dInfo),
+        hipsolver_orgqr_ungqr(API, handle, m, n, k, (T) nullptr, lda, dIpiv, dWork, lwork, dInfo),
         HIPSOLVER_STATUS_INVALID_VALUE);
     EXPECT_ROCBLAS_STATUS(
-        hipsolver_orgqr_ungqr(FORTRAN, handle, m, n, k, dA, lda, (T) nullptr, dWork, lwork, dInfo),
+        hipsolver_orgqr_ungqr(API, handle, m, n, k, dA, lda, (T) nullptr, dWork, lwork, dInfo),
         HIPSOLVER_STATUS_INVALID_VALUE);
     EXPECT_ROCBLAS_STATUS(
-        hipsolver_orgqr_ungqr(FORTRAN, handle, m, n, k, dA, lda, dIpiv, dWork, lwork, (U) nullptr),
+        hipsolver_orgqr_ungqr(API, handle, m, n, k, dA, lda, dIpiv, dWork, lwork, (U) nullptr),
         HIPSOLVER_STATUS_INVALID_VALUE);
 #endif
 }
 
-template <bool FORTRAN, typename T>
+template <testAPI_t API, typename T>
 void testing_orgqr_ungqr_bad_arg()
 {
     // safe arguments
@@ -79,14 +78,13 @@ void testing_orgqr_ungqr_bad_arg()
     CHECK_HIP_ERROR(dInfo.memcheck());
 
     int size_W;
-    hipsolver_orgqr_ungqr_bufferSize(
-        FORTRAN, handle, m, n, k, dA.data(), lda, dIpiv.data(), &size_W);
+    hipsolver_orgqr_ungqr_bufferSize(API, handle, m, n, k, dA.data(), lda, dIpiv.data(), &size_W);
     device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
     if(size_W)
         CHECK_HIP_ERROR(dWork.memcheck());
 
     // check bad arguments
-    orgqr_ungqr_checkBadArgs<FORTRAN>(
+    orgqr_ungqr_checkBadArgs<API>(
         handle, m, n, k, dA.data(), lda, dIpiv.data(), dWork.data(), size_W, dInfo.data());
 }
 
@@ -122,7 +120,7 @@ void orgqr_ungqr_initData(const hipsolverHandle_t handle,
         }
 
         // compute QR factorization
-        cblas_geqrf<T>(m, n, hA[0], lda, hIpiv[0], hW.data(), size_W, &info);
+        cpu_geqrf(m, n, hA[0], lda, hIpiv[0], hW.data(), size_W, &info);
     }
 
     if(GPU)
@@ -133,7 +131,7 @@ void orgqr_ungqr_initData(const hipsolverHandle_t handle,
     }
 }
 
-template <bool FORTRAN, typename T, typename Td, typename Ud, typename Th, typename Uh>
+template <testAPI_t API, typename T, typename Td, typename Ud, typename Th, typename Uh>
 void orgqr_ungqr_getError(const hipsolverHandle_t handle,
                           const int               m,
                           const int               n,
@@ -160,12 +158,12 @@ void orgqr_ungqr_getError(const hipsolverHandle_t handle,
     // execute computations
     // GPU lapack
     CHECK_ROCBLAS_ERROR(hipsolver_orgqr_ungqr(
-        FORTRAN, handle, m, n, k, dA.data(), lda, dIpiv.data(), dWork.data(), lwork, dInfo.data()));
+        API, handle, m, n, k, dA.data(), lda, dIpiv.data(), dWork.data(), lwork, dInfo.data()));
     CHECK_HIP_ERROR(hARes.transfer_from(dA));
     CHECK_HIP_ERROR(hInfoRes.transfer_from(dInfo));
 
     // CPU lapack
-    cblas_orgqr_ungqr<T>(m, n, k, hA[0], lda, hIpiv[0], hW.data(), size_W, hInfo[0]);
+    cpu_orgqr_ungqr(m, n, k, hA[0], lda, hIpiv[0], hW.data(), size_W, hInfo[0]);
 
     // error is ||hA - hAr|| / ||hA||
     // (THIS DOES NOT ACCOUNT FOR NUMERICAL REPRODUCIBILITY ISSUES.
@@ -179,7 +177,7 @@ void orgqr_ungqr_getError(const hipsolverHandle_t handle,
         *max_err += 1;
 }
 
-template <bool FORTRAN, typename T, typename Td, typename Ud, typename Th, typename Uh>
+template <testAPI_t API, typename T, typename Td, typename Ud, typename Th, typename Uh>
 void orgqr_ungqr_getPerfData(const hipsolverHandle_t handle,
                              const int               m,
                              const int               n,
@@ -208,7 +206,7 @@ void orgqr_ungqr_getPerfData(const hipsolverHandle_t handle,
 
         // cpu-lapack performance (only if not in perf mode)
         *cpu_time_used = get_time_us_no_sync();
-        cblas_orgqr_ungqr<T>(m, n, k, hA[0], lda, hIpiv[0], hW.data(), size_W, hInfo[0]);
+        cpu_orgqr_ungqr(m, n, k, hA[0], lda, hIpiv[0], hW.data(), size_W, hInfo[0]);
         *cpu_time_used = get_time_us_no_sync() - *cpu_time_used;
     }
 
@@ -220,17 +218,8 @@ void orgqr_ungqr_getPerfData(const hipsolverHandle_t handle,
         orgqr_ungqr_initData<false, true, T>(
             handle, m, n, k, dA, lda, dIpiv, hA, hIpiv, hW, size_W);
 
-        CHECK_ROCBLAS_ERROR(hipsolver_orgqr_ungqr(FORTRAN,
-                                                  handle,
-                                                  m,
-                                                  n,
-                                                  k,
-                                                  dA.data(),
-                                                  lda,
-                                                  dIpiv.data(),
-                                                  dWork.data(),
-                                                  lwork,
-                                                  dInfo.data()));
+        CHECK_ROCBLAS_ERROR(hipsolver_orgqr_ungqr(
+            API, handle, m, n, k, dA.data(), lda, dIpiv.data(), dWork.data(), lwork, dInfo.data()));
     }
 
     // gpu-lapack performance
@@ -244,23 +233,14 @@ void orgqr_ungqr_getPerfData(const hipsolverHandle_t handle,
             handle, m, n, k, dA, lda, dIpiv, hA, hIpiv, hW, size_W);
 
         start = get_time_us_sync(stream);
-        hipsolver_orgqr_ungqr(FORTRAN,
-                              handle,
-                              m,
-                              n,
-                              k,
-                              dA.data(),
-                              lda,
-                              dIpiv.data(),
-                              dWork.data(),
-                              lwork,
-                              dInfo.data());
+        hipsolver_orgqr_ungqr(
+            API, handle, m, n, k, dA.data(), lda, dIpiv.data(), dWork.data(), lwork, dInfo.data());
         *gpu_time_used += get_time_us_sync(stream) - start;
     }
     *gpu_time_used /= hot_calls;
 }
 
-template <bool FORTRAN, typename T>
+template <testAPI_t API, typename T>
 void testing_orgqr_ungqr(Arguments& argus)
 {
     // get arguments
@@ -286,18 +266,10 @@ void testing_orgqr_ungqr(Arguments& argus)
     bool invalid_size = (m < 0 || n < 0 || k < 0 || lda < m || n > m || k > n);
     if(invalid_size)
     {
-        EXPECT_ROCBLAS_STATUS(hipsolver_orgqr_ungqr(FORTRAN,
-                                                    handle,
-                                                    m,
-                                                    n,
-                                                    k,
-                                                    (T*)nullptr,
-                                                    lda,
-                                                    (T*)nullptr,
-                                                    (T*)nullptr,
-                                                    0,
-                                                    (int*)nullptr),
-                              HIPSOLVER_STATUS_INVALID_VALUE);
+        EXPECT_ROCBLAS_STATUS(
+            hipsolver_orgqr_ungqr(
+                API, handle, m, n, k, (T*)nullptr, lda, (T*)nullptr, (T*)nullptr, 0, (int*)nullptr),
+            HIPSOLVER_STATUS_INVALID_VALUE);
 
         if(argus.timing)
             rocsolver_bench_inform(inform_invalid_size);
@@ -307,8 +279,7 @@ void testing_orgqr_ungqr(Arguments& argus)
 
     // memory size query is necessary
     int size_W;
-    hipsolver_orgqr_ungqr_bufferSize(
-        FORTRAN, handle, m, n, k, (T*)nullptr, lda, (T*)nullptr, &size_W);
+    hipsolver_orgqr_ungqr_bufferSize(API, handle, m, n, k, (T*)nullptr, lda, (T*)nullptr, &size_W);
 
     if(argus.mem_query)
     {
@@ -336,42 +307,42 @@ void testing_orgqr_ungqr(Arguments& argus)
 
     // check computations
     if(argus.unit_check || argus.norm_check)
-        orgqr_ungqr_getError<FORTRAN, T>(handle,
-                                         m,
-                                         n,
-                                         k,
-                                         dA,
-                                         lda,
-                                         dIpiv,
-                                         dWork,
-                                         size_W,
-                                         dInfo,
-                                         hA,
-                                         hARes,
-                                         hIpiv,
-                                         hInfo,
-                                         hInfoRes,
-                                         &max_error);
+        orgqr_ungqr_getError<API, T>(handle,
+                                     m,
+                                     n,
+                                     k,
+                                     dA,
+                                     lda,
+                                     dIpiv,
+                                     dWork,
+                                     size_W,
+                                     dInfo,
+                                     hA,
+                                     hARes,
+                                     hIpiv,
+                                     hInfo,
+                                     hInfoRes,
+                                     &max_error);
 
     // collect performance data
     if(argus.timing)
-        orgqr_ungqr_getPerfData<FORTRAN, T>(handle,
-                                            m,
-                                            n,
-                                            k,
-                                            dA,
-                                            lda,
-                                            dIpiv,
-                                            dWork,
-                                            size_W,
-                                            dInfo,
-                                            hA,
-                                            hIpiv,
-                                            hInfo,
-                                            &gpu_time_used,
-                                            &cpu_time_used,
-                                            hot_calls,
-                                            argus.perf);
+        orgqr_ungqr_getPerfData<API, T>(handle,
+                                        m,
+                                        n,
+                                        k,
+                                        dA,
+                                        lda,
+                                        dIpiv,
+                                        dWork,
+                                        size_W,
+                                        dInfo,
+                                        hA,
+                                        hIpiv,
+                                        hInfo,
+                                        &gpu_time_used,
+                                        &cpu_time_used,
+                                        hot_calls,
+                                        argus.perf);
 
     // validate results for rocsolver-test
     // using m * machine_precision as tolerance
