@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,6 +29,7 @@
 #include "exceptions.hpp"
 #include "hipsolver.h"
 #include "hipsolver_conversions.hpp"
+#include "lib_macros.hpp"
 
 #include <cusolverSp.h>
 
@@ -41,11 +42,11 @@ try
     if(!handle)
         return HIPSOLVER_STATUS_NOT_INITIALIZED;
 
-    return cuda2hip_status(cusolverSpCreate((cusolverSpHandle_t*)handle));
+    return hipsolver::cuda2hip_status(cusolverSpCreate((cusolverSpHandle_t*)handle));
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }
 
 hipsolverStatus_t hipsolverSpDestroy(hipsolverSpHandle_t handle)
@@ -54,11 +55,11 @@ try
     if(!handle)
         return HIPSOLVER_STATUS_NOT_INITIALIZED;
 
-    return cuda2hip_status(cusolverSpDestroy((cusolverSpHandle_t)handle));
+    return hipsolver::cuda2hip_status(cusolverSpDestroy((cusolverSpHandle_t)handle));
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }
 
 hipsolverStatus_t hipsolverSpSetStream(hipsolverSpHandle_t handle, hipStream_t streamId)
@@ -67,11 +68,12 @@ try
     if(!handle)
         return HIPSOLVER_STATUS_NOT_INITIALIZED;
 
-    return cuda2hip_status(cusolverSpSetStream((cusolverSpHandle_t)handle, (cudaStream_t)streamId));
+    return hipsolver::cuda2hip_status(
+        cusolverSpSetStream((cusolverSpHandle_t)handle, (cudaStream_t)streamId));
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }
 
 /******************** CSRLSVCHOL ********************/
@@ -94,22 +96,22 @@ try
     if(!descrA)
         return HIPSOLVER_STATUS_INVALID_VALUE;
 
-    return cuda2hip_status(cusolverSpScsrlsvchol((cusolverSpHandle_t)handle,
-                                                 n,
-                                                 nnzA,
-                                                 (cusparseMatDescr_t)descrA,
-                                                 csrVal,
-                                                 csrRowPtr,
-                                                 csrColInd,
-                                                 b,
-                                                 tolerance,
-                                                 reorder,
-                                                 x,
-                                                 singularity));
+    return hipsolver::cuda2hip_status(cusolverSpScsrlsvchol((cusolverSpHandle_t)handle,
+                                                            n,
+                                                            nnzA,
+                                                            (cusparseMatDescr_t)descrA,
+                                                            csrVal,
+                                                            csrRowPtr,
+                                                            csrColInd,
+                                                            b,
+                                                            tolerance,
+                                                            reorder,
+                                                            x,
+                                                            singularity));
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }
 
 hipsolverStatus_t hipsolverSpDcsrlsvchol(hipsolverSpHandle_t       handle,
@@ -131,22 +133,22 @@ try
     if(!descrA)
         return HIPSOLVER_STATUS_INVALID_VALUE;
 
-    return cuda2hip_status(cusolverSpDcsrlsvchol((cusolverSpHandle_t)handle,
-                                                 n,
-                                                 nnzA,
-                                                 (cusparseMatDescr_t)descrA,
-                                                 csrVal,
-                                                 csrRowPtr,
-                                                 csrColInd,
-                                                 b,
-                                                 tolerance,
-                                                 reorder,
-                                                 x,
-                                                 singularity));
+    return hipsolver::cuda2hip_status(cusolverSpDcsrlsvchol((cusolverSpHandle_t)handle,
+                                                            n,
+                                                            nnzA,
+                                                            (cusparseMatDescr_t)descrA,
+                                                            csrVal,
+                                                            csrRowPtr,
+                                                            csrColInd,
+                                                            b,
+                                                            tolerance,
+                                                            reorder,
+                                                            x,
+                                                            singularity));
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }
 
 /*hipsolverStatus_t hipsolverSpCcsrlsvchol(hipsolverSpHandle_t       handle,
@@ -168,7 +170,7 @@ try
     if(!descrA)
         return HIPSOLVER_STATUS_INVALID_VALUE;
 
-    return cuda2hip_status(cusolverSpCcsrlsvchol((cusolverSpHandle_t)handle,
+    return hipsolver::cuda2hip_status(cusolverSpCcsrlsvchol((cusolverSpHandle_t)handle,
                                                  n,
                                                  nnzA,
                                                  (cusparseMatDescr_t)descrA,
@@ -183,7 +185,7 @@ try
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }
 
 hipsolverStatus_t hipsolverSpZcsrlsvchol(hipsolverSpHandle_t       handle,
@@ -205,7 +207,7 @@ try
     if(!descrA)
         return HIPSOLVER_STATUS_INVALID_VALUE;
 
-    return cuda2hip_status(cusolverSpZcsrlsvchol((cusolverSpHandle_t)handle,
+    return hipsolver::cuda2hip_status(cusolverSpZcsrlsvchol((cusolverSpHandle_t)handle,
                                                  n,
                                                  nnzA,
                                                  (cusparseMatDescr_t)descrA,
@@ -220,7 +222,7 @@ try
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }*/
 
 hipsolverStatus_t hipsolverSpScsrlsvcholHost(hipsolverSpHandle_t       handle,
@@ -242,22 +244,22 @@ try
     if(!descrA)
         return HIPSOLVER_STATUS_INVALID_VALUE;
 
-    return cuda2hip_status(cusolverSpScsrlsvcholHost((cusolverSpHandle_t)handle,
-                                                     n,
-                                                     nnzA,
-                                                     (cusparseMatDescr_t)descrA,
-                                                     csrVal,
-                                                     csrRowPtr,
-                                                     csrColInd,
-                                                     b,
-                                                     tolerance,
-                                                     reorder,
-                                                     x,
-                                                     singularity));
+    return hipsolver::cuda2hip_status(cusolverSpScsrlsvcholHost((cusolverSpHandle_t)handle,
+                                                                n,
+                                                                nnzA,
+                                                                (cusparseMatDescr_t)descrA,
+                                                                csrVal,
+                                                                csrRowPtr,
+                                                                csrColInd,
+                                                                b,
+                                                                tolerance,
+                                                                reorder,
+                                                                x,
+                                                                singularity));
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }
 
 hipsolverStatus_t hipsolverSpDcsrlsvcholHost(hipsolverSpHandle_t       handle,
@@ -279,22 +281,22 @@ try
     if(!descrA)
         return HIPSOLVER_STATUS_INVALID_VALUE;
 
-    return cuda2hip_status(cusolverSpDcsrlsvcholHost((cusolverSpHandle_t)handle,
-                                                     n,
-                                                     nnzA,
-                                                     (cusparseMatDescr_t)descrA,
-                                                     csrVal,
-                                                     csrRowPtr,
-                                                     csrColInd,
-                                                     b,
-                                                     tolerance,
-                                                     reorder,
-                                                     x,
-                                                     singularity));
+    return hipsolver::cuda2hip_status(cusolverSpDcsrlsvcholHost((cusolverSpHandle_t)handle,
+                                                                n,
+                                                                nnzA,
+                                                                (cusparseMatDescr_t)descrA,
+                                                                csrVal,
+                                                                csrRowPtr,
+                                                                csrColInd,
+                                                                b,
+                                                                tolerance,
+                                                                reorder,
+                                                                x,
+                                                                singularity));
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }
 
 /*hipsolverStatus_t hipsolverSpCcsrlsvcholHost(hipsolverSpHandle_t       handle,
@@ -316,7 +318,7 @@ try
     if(!descrA)
         return HIPSOLVER_STATUS_INVALID_VALUE;
 
-    return cuda2hip_status(cusolverSpCcsrlsvcholHost((cusolverSpHandle_t)handle,
+    return hipsolver::cuda2hip_status(cusolverSpCcsrlsvcholHost((cusolverSpHandle_t)handle,
                                                      n,
                                                      nnzA,
                                                      (cusparseMatDescr_t)descrA,
@@ -331,7 +333,7 @@ try
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }
 
 hipsolverStatus_t hipsolverSpZcsrlsvcholHost(hipsolverSpHandle_t       handle,
@@ -353,7 +355,7 @@ try
     if(!descrA)
         return HIPSOLVER_STATUS_INVALID_VALUE;
 
-    return cuda2hip_status(cusolverSpZcsrlsvcholHost((cusolverSpHandle_t)handle,
+    return hipsolver::cuda2hip_status(cusolverSpZcsrlsvcholHost((cusolverSpHandle_t)handle,
                                                      n,
                                                      nnzA,
                                                      (cusparseMatDescr_t)descrA,
@@ -368,7 +370,7 @@ try
 }
 catch(...)
 {
-    return exception2hip_status();
+    return hipsolver::exception2hip_status();
 }*/
 
 } //extern C
